require 'minitest/autorun'

require 'rubenum'

class TestRubenum < Minitest::Test
	# Valid elements

	def test_with_valid_elements
		e = Rubenum.new(%w[First sec_ond third_ fou_rth4])
		assert_equal e.constants.sort, [:First, :Sec_ond, :Third_, :Fou_rth4].sort
	end

	def test_with_a_single_char
		e = Rubenum.new(%w[First x sec_ond third_ fou_rth4])
		assert_equal e.constants.sort, [:First, :X, :Sec_ond, :Third_, :Fou_rth4].sort
	end

	def test_duplicate_elements_are_made_unique
		e = Rubenum.new(%w[First first sec_ond third_ fou_rth4])
		assert_equal e.constants.sort, [:First, :Sec_ond, :Third_, :Fou_rth4].sort
	end

	# Invalid elements

	def test_when_elements_isnt_an_array
		assert_raises ArgumentError do
			Rubenum.new(45)
		end
	end

	def test_with_empty_array
		assert_raises ArgumentError do
			Rubenum.new([])
		end
	end

	def test_with_wrong_type_of_elements
		assert_raises ArgumentError do
			Rubenum.new([:First, "sec_ond", ["third"], 4])
		end
	end

	def test_not_working_with_a_space_char
		assert_raises NameError do
			Rubenum.new(["First", "sec ond", "third_", "fou_rth4"])
		end
	end

	def test_not_working_with_a_special_char
		assert_raises NameError do
			Rubenum.new(%w[First sec!ond third_ fou_rth4])
		end
	end

	def test_not_working_with_a_leading_underscore
		assert_raises NameError do
			Rubenum.new(%w[First _sec_ond third_ fou_rth4])
		end
	end

	def test_not_working_with_a_leading_digit
		assert_raises NameError do
			Rubenum.new(%w[First 2sec_ond third_ fou_rth4])
		end
	end

	# Upcase option

	def test_upcase
		e = Rubenum.new(%w[First sec_ond third_ fou_rth4], upcase: true)
		assert_equal e.constants.sort, [:FIRST, :SEC_OND, :THIRD_, :FOU_RTH4].sort
	end

	# Binary values option

	def test_with_decimal
		e = Rubenum.new(%w[First sec_ond third_ fou_rth4])
		assert_equal 1, e::First
		assert_equal 2, e::Sec_ond
		assert_equal 3, e::Third_
		assert_equal 4, e::Fou_rth4
	end

	def test_with_binary
		e = Rubenum.new(%w[First sec_ond third_ fou_rth4], binary_values: true)
		assert_equal 1, e::First
		assert_equal 2, e::Sec_ond
		assert_equal 4, e::Third_
		assert_equal 8, e::Fou_rth4
	end
end

